
open Dns
open Dns_txt
module Misc = Muthr_misc

type nserver =
    Glue of Unix.inet_addr
  | Extern of name

let string_of_auth right (n, ttl, l) =
  Printf.sprintf (if right then format_of_string "%23s %6i %s" else format_of_string "%-23s %i %s")
    (string_of_name n) ttl
    (String.concat " " (List.map (fun e -> match e with
    | Glue ip -> Unix.string_of_inet_addr ip
    | Extern n -> string_of_name n) l))


type answer_t =

    (* final answers *)
  | Data of     int * rdset_t
  | DataAny of (int * rdset_t) list
  | NoData  of  int * name
  | NxDomain of int * name
  | RFailed of string

    (* partial answers *)
  | Referral of name * rrset_t * rrset_t list
  | Alias of name

    (* ``bad answers'' *)
  | Error   of int
  | Truncated
  | Lame    of name
  | Invalid of string


(* ZZZ *)
let string_of_rrset (o, t, rd) = 
  Printf.sprintf "%s %i %s" (string_of_name o) t (string_of_rdset rd)

let string_of_rrsets rrs =
  String.concat "\n" (List.map string_of_rrset rrs)

let string_of_answer_t a =
  match a with
  | Data(ttl, rr) -> Printf.sprintf "Data %i %s" ttl (string_of_rdset_sl rr)
  | DataAny(rrs)  -> String.concat "\n" (List.map 
    (fun (ttl,rr) -> Printf.sprintf "%i %s" ttl (string_of_rdset_sl rr)) rrs)
  | NoData(ttl,_)   -> Printf.sprintf "%i NoData" ttl
  | NxDomain(ttl,_) -> Printf.sprintf "%i NxDomain" ttl
  | RFailed s       -> "RFailed " ^ s

  | Referral(qn, ns, add) -> Printf.sprintf "Referral %s %s\n%s"
	(string_of_name qn) (string_of_rrset ns) (string_of_rrsets add)
  | Alias n       -> Printf.sprintf "Alias %s" (string_of_name n)

  | Error i       -> Printf.sprintf "* Error %i *" i
  | Truncated     -> "Truncated"
  | Lame n        -> Printf.sprintf "Lame: %s" (string_of_name n)
  | Invalid s     -> Printf.sprintf "* Invalid %s *" s


let debug = ref false

let dbg s = if !debug then Misc.log s

let make_auth aut add =
  let owner, ttl, names = match aut with owner, ttl, NS ns -> owner, ttl, ns
  in
  let glue, ext = List.fold_left 
      (fun (gl,ex) n -> try
	let (_,ttl, A ips) = List.find (fun (n', _, rd) -> n=n'&& rrt_of_rdset_t rd = rrt_a) add in
	((List.hd ips) :: gl, ex) (* ZZZ only first IP *)
      with Not_found ->
	(gl, n :: ex))
      ([],[])
      names
  in
  let ttl = List.fold_left min ttl (List.map (fun (_,t,_) -> t) add)
  in
  let l1 = List.map (fun ip -> Glue  ip) glue in
  let l2 = List.map (fun n  -> Extern n) ext  in
  owner, ttl, l1 @ l2
    
