(* TODO: ip adr, good port ?
   see if simpler "hand-made"
   open existing file
*)

open Pcap

(* link level header for LINKTYPE_NULL=0 *)
let llh = "\000\000\008\000"

let ip =
  let s = String.create 20 in
  let t = [|
    0x45;0x00;0x00;0x00;0x00;0x00;0x40;0x00;0x40;0x11;0x00;0x00;
    0x00;0x00;0x00;0x01;0x00;0x00;0x00;0x00 |]
  in
  Array.iteri (fun i b -> String.set s i (char_of_int b)) t;
  s

let myaddr = "\000\000\000\000"

let put_16 b o n =
  b.[o]   <- char_of_int ((n land 0xFF00) lsr 8);
  b.[o+1] <- char_of_int  (n land 0x00FF)

let code_of_error e =
  match e with
  | Dns.BadClass _  -> 0xbadC
  | Dns.BadRdata _  -> 0xbadD
  | Dns.BadFormat _ -> 0xbadF

let save_msg d snd m =
  let Some(e,b) = m.Dns.bad in
  let adr  = m.Dns.adr in
  let port = m.Dns.port in
  let len  = m.Dns.len
  in
  let t = Unix.gettimeofday () in
  let s = int_of_float t in
  let us = int_of_float ((t -. (float_of_int s)) *. 1000000.) in
  let ts  = { tv_sec = s; tv_usec = us }
  in
  let iplen = len + 28 in
  let hdr = { ts=ts; caplen=1024; len=iplen+4}
  in
  put_16 ip 2 iplen;
  put_16 ip 10 (code_of_error e);
  String.blit (if snd then myaddr else (Obj.magic adr)) 0 ip 12 4;
  String.blit (if snd then (Obj.magic adr) else myaddr) 0 ip 16 4;
  let udp = String.make 8 '\000' in
  put_16 udp (if snd then 0 else 2) 53;
  put_16 udp (if snd then 2 else 0) port;
  put_16 udp 4 (len+8);
  pcap_dump_direct d hdr (llh ^ ip ^ udp ^ (String.sub b 0 len))

let close_file d =
  pcap_dump_close d

let open_file f =
  let h = pcap_open_dead 0 1024 in
  let d = pcap_dump_open h f in
  (save_msg d), (fun () -> close_file d)
