open Dns
open Dns_txt

(*s dinggy: a minimal dig-like DNS querying tool *)

let msg_size = 1024

let resolv_conf = "/etc/resolv.conf"

let resolvers = ref []

let max_tries = ref 3
let request_tm = ref 5.

let read_conf () =
  let f = open_in resolv_conf
  in
  let rec loop acc =
    try
      let l = input_line f in
      Scanf.sscanf l "%s %s" (fun key value ->
      loop (if key="nameserver" then (Unix.inet_addr_of_string value)::acc else acc))
    with
      End_of_file -> close_in f; acc
  in
  resolvers := List.rev (loop [])


let dns_recv fd =
  let b = String.create msg_size in
  let (l,Unix.ADDR_INET(adr,port)) = Unix.recvfrom fd b 0 msg_size [] in
  let m = decode_msg b l adr port in
  m
    
let dns_send fd m =
  let l, b = encode_msg m in
  Unix.sendto fd b 0 l [] 
    (Unix.ADDR_INET (m.adr, m.port))



let make_query ip sname stype sclass size =
  make_query ip (Random.int 64511 + 1024) true sname stype sclass None
 (*Some(size,0,"");*)

let do_resolv sname stype =
  let fd = Unix.socket Unix.PF_INET Unix.SOCK_DGRAM 0 in
  let m  = make_query (List.hd !resolvers) sname stype 1 msg_size
  in
  print_string "> sending query\n";
  print_string (string_of_msg m);
  dns_send fd m;
  let sent = Unix.gettimeofday () 
  in
  let m = dns_recv fd in
  let rcvd = Unix.gettimeofday ()
  in
  Printf.printf "\n< received DNS message (%i ms)\n" (int_of_float ((rcvd-.sent) *. 1000.));
  print_string (string_of_msg m)


let ding qn qt qc =
  do_resolv (name_of_string qn) qt

let set_resolver sip =
  resolvers := Unix.inet_addr_of_string sip :: !resolvers


let qtype  = ref rrt_a
let qclass = ref 1

let spec = [("-r", Arg.String set_resolver,   "resolver IP");
            ("-t", Arg.String (fun s -> qtype := rrt_of_string s), "query type");
	    ("-m", Arg.Set_int    max_tries,  "number of tries");
	    ("-tm", Arg.Set_float  request_tm, "retransmission timeout");
	  ]
let usage_msg = "dinggy [options] name."


let main () =
  let n = ref "" in
  Random.self_init();
  read_conf ();
  Arg.parse spec
    (fun na -> n := na)
    usage_msg;
  ding !n !qtype !qclass
;;
main ()
