
(* man 5 pcap-savefile, 7 pcap-linktype *)

let global_header =
  let magic_number = [ 0xa1; 0xb2; 0xc3; 0xd4] in 
  let version_major = [ 0; 2 ] in
  let version_minor = [ 0; 4 ] in
  let this_zone = [ 0;0;0;0 ] in (* time zone offset, always 0 *)
  let sigfigs   = [ 0;0;0;0 ] in (* timestamp accuracy, always 0 *)
  let snaplen   = [ 0;0;4;0 ] in (* 1024 *)
  let dl_type   = [ 0;0;0;113 ]  (* linux "cooked" capture ZZZ usr raw?=101 *)
  in
  magic_number @ version_major @ version_minor @ this_zone
  @ sigfigs @ snaplen @ dl_type

(* + date *)
let packet_header len =
  [ 0; 0; 0; 0;   (* timestamp, seconds value *)
    0; 0; 0; 0;   (* timestamp, microseconds value *)
    0; 0; 0; len; (* length of capture packet data *)
    0; 0; 0; len ](* un-truncated length of the packet data *)

let sll_header = (* 16 bytes *)
  let packet_type = [0;0] in
  let arphdr      = [0;0] in
  let lla_len     = [0;0] in
  let lla         = [0;0;0;0;0;0;0;0] in
  let protocol    = [0x8; 0]
  in
  packet_type @ arphdr @ lla_len @ lla @ protocol

(* + source ip *)
let ip_header hlen llen = (* 20 bytes *)
  [
  0x45;0x00;hlen;llen;0x00;0x00;0x40;0x00;0x40;0x11;0x5b;0xd8;
  0xc0;0xa8;0x00;0x01;0xc0;0x0c;0x5e;0x1e ]

(* + source port *)
let udp_header = (* 8 bytes *)
  [ 0xcf;0x95;0x00;0x35;0x00;0x2d;0xdf;0x12 ]

let dns_packet = [
  0x23;0x70;0x00;0x00;0x00;0x01;0x00;0x00;0x00;0x00;0x00;0x01;0x04;0x72;0x65;0x7a;0x6f;0x03;0x6e;0x65;0x74;0x00;0x00;0x1c;0x00;0x01;0x00;0x00;0x29;0x04;0x00;0x00;0x00;0x00;0x00;0x00;0x00]

let open_file n =
  let f = open_out n in
  List.iter (fun c -> output_byte f c) global_header;
  f

let write_file f dns =
  let iplen = 20 + 8 + List.length dns in
  let len   = iplen + 16
  in
  List.iter
    (fun c -> output_byte f c)
    (packet_header len @ sll_header @
     ip_header (iplen / 256) (iplen mod 256) @ udp_header @ dns)

let close_file f =
  close_out f
