open Muthr
open Dns
open Dns_txt
module Misc = Muthr_misc

let log s =
  Misc.log ("W: " ^ s)

let log_n n s =
  Misc.log (Printf.sprintf "W: %s -> %s" (string_of_name n) s)


let error s = Printf.fprintf stderr "%s\n" s; exit 1

let msg_size     = 1024
let max_tries    = ref 3
let request_tm   = ref 1.
let mretry_delay = ref 0.5


let ssq = ref 0  (* sent queries *)
let ssr = ref 0  (* sent replies *)
let srq = ref 0  (* received queries *)
let srr = ref 0  (* received replies *)

let sbq = ref 0  (* bad queries received *)
let sbr = ref 0  (* bad replies received *)


let ser = ref 0  (* number of external resolutions *)
let sir = ref 0  (* number of internal resolutions *)


let dbg = Answer.dbg

let debug_msg = ref false
let dbg_msg t m =
  if !debug_msg then 
    Misc.log ("M: " ^ Printf.sprintf "%s\n%s" t (string_of_msg m))



let new_id =
  let c = ref 0 in
  fun () -> incr c; !c

type state = {
    mutable sn : name; st : rrt; sc : int;
    id : int;  parents : int list;
  }(* + acc ? + zone ? *)

let ext_state sn st =
  { sn = sn; st = st; sc = 1; id = new_id (); parents = [] }

let int_state sn st ste =
  { sn = sn; st = st; sc = 1; id = new_id (); 
    parents = ste.id :: ste.parents }


let dbg_n t n = dbg (t ^ " " ^ (string_of_name n))
let dbg_st t1 ste t2 = dbg (Printf.sprintf "%s .%i. %s %s %s" t1 ste.id (string_of_name ste.sn) (string_of_rrt ste.st) t2)
let dbg_nz t n z = dbg (t ^ " " ^ (string_of_name n) ^ " (zone " ^ (string_of_name z) ^")")

let dbg_msg2 t = if !debug_msg then log t



(*s Sending and receiving messages *)

let dns_recv fd k =
  let b = String.create msg_size in
  recvfrom fd b 0 msg_size [] >>= fun (l,Unix.ADDR_INET(adr,port)) -> 
  let m = decode_msg b l adr port in
  k m

let dns_send_raw fd ip b =
  incr ssq;
  ignore (Unix.sendto fd b 0 (String.length b) [] 
	    (Unix.ADDR_INET (ip, 53)))
  
let dns_send fd m =
  let l, b = encode_msg m in
  ignore (Unix.sendto fd b 0 l [] 
	    (Unix.ADDR_INET (m.adr, m.port)))

let dns_send_reply fd m =
  incr ssr;
  dns_send fd m

let dns_send_query fd m =
  incr ssq;
  dns_send fd m
